@@ -40,7 +40,7 @@ package canu::Defaults;
 require Exporter;
 
 @ISA    = qw(Exporter);
-@EXPORT = qw(getCommandLineOptions addCommandLineOption addCommandLineError writeLog getNumberOfCPUs getPhysicalMemorySize getAllowedResources diskSpace printOptions printHelp printCitation addSequenceFile setParametersFromFile setParametersFromCommandLine checkJava checkGnuplot checkParameters getGlobal setGlobal setGlobalIfUndef setDefaults setVersion);
+@EXPORT = qw(getCommandLineOptions addCommandLineOption addCommandLineError writeLog getNumberOfCPUs getPhysicalMemorySize diskSpace printOptions printHelp printCitation addSequenceFile setParametersFromFile setParametersFromCommandLine checkJava checkGnuplot checkParameters getGlobal setGlobal setGlobalIfUndef setDefaults setVersion);
 
 use strict;
 use Cwd qw(getcwd abs_path);
@@ -133,7 +133,6 @@ sub setGlobal ($$) {
 
     foreach my $opt ("ovlerrorrate",
                      "ovlhashblocklength",
-                     "ovlrefblocksize",
                      "ovlrefblocklength",
                      "ovlhashbits",
                      "ovlhashload",
@@ -407,9 +406,9 @@ sub printHelp (@) {
     print "  an optional sepc file with the -s option.\n";
     print "\n";
     print "  Reads can be either FASTA or FASTQ format, uncompressed, or compressed with gz, bz2 or xz.\n";
-    print "  Reads are specified by the technology they were generated with:\n";
-    print "    -pacbio-raw         <files>\n";
-    print "    -pacbio-corrected   <files>\n";
+    print "  Reads are specified by the technology they were generated with, and any processing performed:\n";
+    print "    -pacbio-raw         <files>      Reads are straight off the machine.\n";
+    print "    -pacbio-corrected   <files>      Reads have been corrected.\n";
     print "    -nanopore-raw       <files>\n";
     print "    -nanopore-corrected <files>\n";
     print "\n";
@@ -509,6 +508,7 @@ sub addSequenceFile ($$@) {
 
     return(undef)             if (!defined($file));   #  No file name?  Nothing to do.
     $file = "$dir/$file"      if (defined($dir));     #  If $dir defined, assume file is in there.
+    return($file)             if (substr($file, 0, 1) eq "/");     #  If already a full path, use that.
     return(abs_path($file))   if (-e $file);          #  If found, return the full path.
 
     #  And if not found, report an error, unless told not to.  This is because on the command
@@ -681,8 +681,7 @@ sub setOverlapDefaults ($$$) {
     #  OverlapInCore parameters.
 
     setOverlapDefault($tag, "OvlHashBlockLength",  undef,                     "Amount of sequence (bp) to load into the overlap hash table");
-    setOverlapDefault($tag, "OvlRefBlockSize",     undef,                     "Number of reads to search against the hash table per batch");
-    setOverlapDefault($tag, "OvlRefBlockLength",   0,                         "Amount of sequence (bp) to search against the hash table per batch");
+    setOverlapDefault($tag, "OvlRefBlockLength",   undef,                     "Amount of sequence (bp) to search against the hash table per batch");
     setOverlapDefault($tag, "OvlHashBits",         ($tag eq "cor") ? 18 : 23, "Width of the kmer hash.  Width 22=1gb, 23=2gb, 24=4gb, 25=8gb.  Plus 10b per ${tag}OvlHashBlockLength");
     setOverlapDefault($tag, "OvlHashLoad",         0.75,                      "Maximum hash table load.  If set too high, table lookups are inefficent; if too low, search overhead dominates run time; default 0.75");
     setOverlapDefault($tag, "OvlMerSize",          ($tag eq "cor") ? 19 : 22, "K-mer size for seeds in overlaps");
@@ -694,7 +693,7 @@ sub setOverlapDefaults ($$$) {
 
     #  Mhap parameters.  FilterThreshold MUST be a string, otherwise it gets printed in scientific notation (5e-06) which java doesn't understand.
 
-    setOverlapDefault($tag, "MhapVersion",         "2.1.2",                   "Version of the MHAP jar file to use");
+    setOverlapDefault($tag, "MhapVersion",         "2.1.3",                   "Version of the MHAP jar file to use");
     setOverlapDefault($tag, "MhapFilterThreshold", "0.000005",                "Value between 0 and 1. kmers which comprise more than this percentage of the input are downweighted");
     setOverlapDefault($tag, "MhapFilterUnique",    undef,                     "Expert option: True or false, supress the low-frequency k-mer distribution based on them being likely noise and not true overlaps. Threshold auto-computed based on error rate and coverage.");
     setOverlapDefault($tag, "MhapNoTf",            undef,                     "Expert option: True or false, do not use tf weighting, only idf of tf-idf.");
@@ -747,19 +746,21 @@ sub setDefaults () {
     my $java = (exists $ENV{"JAVA_HOME"} && -e "$ENV{'JAVA_HOME'}/bin/java") ? "$ENV{'JAVA_HOME'}/bin/java" : "java";
 
     setDefault("showNext",            undef,     "Don't run any commands, just report what would run");
-    setDefault("pathMap",             undef,     "File with a hostname to binary directory map; binary directories must be absolute paths");
     setDefault("shell",               "/bin/sh", "Command interpreter to use; sh-compatible (e.g., bash), NOT C-shell (csh or tcsh); default '/bin/sh'");
     setDefault("java",                $java,     "Java interpreter to use; at least version 1.8; default 'java'");
     setDefault("gnuplot",             "gnuplot", "Path to the gnuplot executable");
-    setDefault("gnuplotImageFormat",  undef,     "Image format that gnuplot will generate, used in HTML reports.  Default: based on gnuplot, 'png', 'svg' or 'gif'");
+    setDefault("gnuplotImageFormat",  undef,     "Image format that gnuplot will generate.  Default: based on gnuplot, 'png', 'svg' or 'gif'");
     setDefault("gnuplotTested",       0,         "If set, skip the initial testing of gnuplot");
     setDefault("stageDirectory",      undef,     "If set, copy heavily used data to this node-local location");
+    setDefault("preExec",             undef,     "A command line to run at the start of Canu execution scripts");
 
     #####  Cleanup and Termination options
 
     setDefault("saveOverlaps",        0,     "Save intermediate overlap files, almost never a good idea");
     setDefault("saveReadCorrections", 0,     "Save intermediate read correction files, almost never a good idea");
+    setDefault("saveReadHaplotypes",  0,     "Save intermediate read haplotype files, almost never a good idea");
     setDefault("saveMerCounts",       0,     "Save full mer counting results, sometimes useful");
+    setDefault("saveReads",           1,     "Save intermediate corrected and trimmed reads to asm.correctedReads.fasta.gz and asm.trimmedReads.fasta.gz");
     setDefault("onSuccess",           undef, "Full path to command to run on successful completion");
     setDefault("onFailure",           undef, "Full path to command to run on failure");
 
@@ -900,7 +901,7 @@ sub setDefaults () {
     setDefault("batThreads",     undef, "Number of threads to use; default is the maxThreads limit");
     setDefault("batConcurrency", undef, "Unused, only one process supported");
 
-    setDefault("contigFilter",   "2 0 1.0 0.5 5",   "Parameters to filter out 'unassembled' unitigs.  Five values: minReads minLength singleReadSpan lowCovFraction lowCovDepth");
+    setDefault("contigFilter",   "2 0 1.0 0.5 3",   "Parameters to filter out 'unassembled' unitigs.  Five values: minReads minLength singleReadSpan lowCovFraction lowCovDepth");
 
     #####  Consensus Options
 
@@ -920,8 +921,7 @@ sub setDefaults () {
     setDefault("corOutCoverage",               40,           "Only correct the longest reads up to this coverage; default 40");
     setDefault("corMinCoverage",               undef,        "Minimum number of bases supporting each corrected base, if less than this sequences are split; default based on input read coverage: 0 <= 30x < 4 < 60x <= 4");
     setDefault("corFilter",                    "expensive",  "Method to filter short reads from correction; 'quick' or 'expensive'; default 'expensive'");
-    setDefault("corConsensus",                 "falconpipe", "Which consensus algorithm to use; only 'falcon' and 'falconpipe' are supported; default 'falconpipe'");
-    setDefault("corLegacyFilter",              undef,        "Expert option: global filter, length * identity (default) or length with  broken by identity (if on)");
+    setDefault("corConsensus",                 "falcon",     "Which consensus algorithm to use; only 'falcon' is supported; default 'falcon'");
 
     #  Convert all the keys to lowercase, and remember the case-sensitive version
 
@@ -947,7 +947,7 @@ sub setDefaults () {
 }
 
 
-#  Get the version information.  Needs to be last so that pathMap can be defined.
+#  Get the version information.
 
 sub setVersion ($) {
     my $bin    = shift @_;
@@ -993,7 +993,7 @@ sub checkJava () {
         addCommandLineError("ERROR:  mhap overlapper requires java version at least 1.8.0; you have $versionStr (from '$java').\n");
         addCommandLineError("ERROR:  '$java -showversion' reports:\n");
 
-        open(F, "$java -showversion 2>&1 |");
+        open(F, "$java -Xmx1g -showversion 2>&1 |");
         while (<F>) {
             chomp;
             addCommandLineError("ERROR:    '$_'\n");
@@ -1265,22 +1265,24 @@ sub checkParameters () {
     }
 
     if ((getGlobal("ovsMethod") ne "sequential") &&
-        (getGlobal("ovsMethod") ne "parallel")) {
-        addCommandLineError("ERROR:  Invalid 'ovsMethod' specified (" . getGlobal("ovsMethod") . "); must be 'sequential' or 'parallel'\n");
+        (getGlobal("ovsMethod") ne "parallel") &&
+        (getGlobal("ovsMethod") ne "forceparallel")) {
+        addCommandLineError("ERROR:  Invalid 'ovsMethod' specified (" . getGlobal("ovsMethod") . "); must be 'sequential', 'parallel', or 'forceparallel' (not recommended)\n");
     }
     if ((getGlobal("useGrid")   eq "0") &&
         (getGlobal("ovsMethod") eq "parallel")) {
-        addCommandLineError("ERROR:  ovsMethod=parallel requires useGrid=true or useGrid=remote.  Set ovsMethod=sequential if no grid is available\n");
+        setGlobal("ovsMethod", "sequential");
+        #addCommandLineError("ERROR:  ovsMethod=parallel requires useGrid=true or useGrid=remote.  Set ovsMethod=sequential if no grid is available, or ovsMethod=forceparallel to use the (disk intensive) parallel method\n");
+    }
+    if (getGlobal("ovsMethod") eq "forceparallel") {
+        setGlobal("ovsMethod", "parallel");
     }
 
-    if ((getGlobal("unitigger") ne "unitigger") &&
-        (getGlobal("unitigger") ne "bogart")) {
+    if ((getGlobal("unitigger") ne "bogart")) {
         addCommandLineError("ERROR:  Invalid 'unitigger' specified (" . getGlobal("unitigger") . "); must be 'unitigger' or 'bogart'\n");
     }
 
-    if ((getGlobal("corConsensus") ne "utgcns") &&
-        (getGlobal("corConsensus") ne "falcon") &&
-        (getGlobal("corConsensus") ne "falconpipe")) {
+    if ((getGlobal("corConsensus") ne "falcon")) {
         addCommandLineError("ERROR:  Invalid 'corConsensus' specified (" . getGlobal("corConsensus") . "); must be 'utgcns' or 'falcon' or 'falconpipe'\n");
     }
 
@@ -1361,14 +1363,6 @@ sub checkParameters () {
         addCommandLineError("contigFilter 'lowCovFraction' must be between 0.0 and 1.0, currently $v[3]\n")   if (($v[3] < 0) || (1 < $v[3]) || ($v[3] !~ m/^[0-9]*\.{0,1}[0-9]*$/));
         addCommandLineError("contigFilter 'lowCovDepth' must be a positive integer, currently $v[4]\n")       if (($v[4] < 0) || ($v[4] !~ m/^[0-9]+$/));
     }
-
-    #
-    #  Minimap, no valid identities, set legacy
-    #
-
-    if (getGlobal("corOverlapper") eq "minimap") {
-        setGlobalIfUndef("corLegacyFilter", 1);
-    }
 }
 
 
