@@ -50,12 +50,60 @@ use canu::Defaults;
 use canu::Execution;
 use canu::Gatekeeper;
 use canu::Report;
-use canu::HTML;
 use canu::Grid_Cloud;
 
 #  Hardcoded to use utgOvlErrorRate
 
 
+sub loadReadLengthsAndNumberOfOverlaps ($$$$) {
+    my $asm     = shift @_;
+    my $maxID   = shift @_;
+    my $rlVec   = shift @_;
+    my $rlSum   = 0;
+    my $noVec   = shift @_;
+    my $noSum   = 0;
+
+    my $bin     = getBinDirectory();
+
+    $$rlVec     = "\xff" x ($maxID * 4 + 4);
+    $$noVec     = "\xff" x ($maxID * 4 + 4);
+
+    print STDERR "--\n";
+    print STDERR "-- Loading read lengths.\n";
+
+    open(F, "$bin/gatekeeperDumpMetaData -G unitigging/$asm.gkpStore -reads |");
+    while (<F>) {
+        s/^\s+//;
+        s/\s+$//;
+        my @v = split '\s+', $_;
+        vec($$rlVec, $v[0], 32) = $v[2];
+        $rlSum                 += $v[2];
+    }
+    close(F);
+
+    caExit("Failed to load read lengths from '$asm.gkpStore'", undef)   if ($rlSum == 0);
+
+
+    fetchStore("unitigging/$asm.ovlStore");
+
+    print STDERR "-- Loading number of overlaps per read.\n";
+
+    open(F, "$bin/ovStoreDump -G unitigging/$asm.gkpStore -O unitigging/$asm.ovlStore -d -counts |");
+    while (<F>) {
+        s/^\s+//;
+        s/\s+$//;
+        my @v = split '\s+', $_;
+        vec($$noVec, $v[0], 32) = $v[1];
+        $noSum                 += $v[1];
+    }
+    close(F);
+
+    caExit("Failed to load number of overlaps per read from '$asm.ovlStore'", undef)   if ($noSum == 0);
+
+    return($rlSum, $noSum);
+}
+
+
 
 sub readErrorDetectionConfigure ($) {
     my $asm     = shift @_;
@@ -74,78 +122,94 @@ sub readErrorDetectionConfigure ($) {
 
     make_path("$path")  if (! -d "$path");
 
-    #  RED uses 13 bytes/base plus 12 bytes/overlap + space for evidence reads.
+    my $maxID = getNumberOfReadsEarliestVersion($asm);
 
-    my @readLengths;
-    my @numOlaps;
+    my ($rlVec, $noVec);
+    my ($rlSum, $noSum) = loadReadLengthsAndNumberOfOverlaps($asm, $maxID, \$rlVec, \$noVec);
 
-    #print STDERR "$bin/gatekeeperDumpMetaData -G unitigging/$asm.gkpStore -reads\n";
-    open(F, "$bin/gatekeeperDumpMetaData -G unitigging/$asm.gkpStore -reads |");
-    while (<F>) {
-        my @v = split '\s+', $_;
-        $readLengths[$v[0]] = $v[2];
+    #  Find the maximum size of each block of 100,000 reads.  findErrors reads up to 100,000 reads
+    #  to process at one time.  It uses 1 * length + 4 * 100,000 bytes of memory for bases and ID storage,
+    #  and has two buffers of this size.
+
+    my $maxBlockSize = 0;
+
+    for (my $id = 1; $id <= $maxID; $id += 100000) {
+        my $sum = 0;
+
+        for (my $ii=$id; ($ii < $id + 100000) && ($ii < $maxID); $ii++) {
+            $sum += vec($rlVec, $ii, 32);
+        }
+
+        $maxBlockSize = $sum   if ($maxBlockSize < $sum);
     }
-    close(F);
 
-    #  NEEDS OPTIMIZE - only need counts here, not the whole store
-
-    fetchStore("unitigging/$asm.ovlStore");
-
-    #print STDERR "$bin/ovStoreDump -G unitigging/$asm.gkpStore -O unitigging/$asm.ovlStore -d -counts\n";
-    open(F, "$bin/ovStoreDump -G unitigging/$asm.gkpStore -O unitigging/$asm.ovlStore -d -counts |");
-    while (<F>) {
-        my @v = split '\s+', $_;
-        $numOlaps[$v[0]] = $v[1];
-    }
-    close(F);
-
-    #  Make an array of partitions, putting as many reads into each as will fit in the desired memory.
-
-    my @bgn;
-    my @end;
-    my $nj = 0;
-
-    #getAllowedResources("", "red");
-
-    my $maxID    = getNumberOfReadsInStore("unitigging", $asm);
     my $maxMem   = getGlobal("redMemory") * 1024 * 1024 * 1024;
     my $maxReads = getGlobal("redBatchSize");
     my $maxBases = getGlobal("redBatchLength");
 
-    print STDERR "\n";
-    print STDERR "Configure RED for ", getGlobal("redMemory"), "gb memory with batches of at most ", ($maxReads > 0) ? $maxReads : "(unlimited)", " reads and ", ($maxBases > 0) ? $maxBases : "(unlimited)", " bases.\n";
-    print STDERR "\n";
+    print STDERR "--\n";
+    print STDERR "-- Configure RED for ", getGlobal("redMemory"), "gb memory.\n";
+    print STDERR "--                   Batches of at most ", ($maxReads > 0) ? $maxReads : "(unlimited)", " reads.\n";
+    print STDERR "--                                      ", ($maxBases > 0) ? $maxBases : "(unlimited)", " bases.\n";
+    print STDERR "--                   Expecting evidence of at most $maxBlockSize bases per iteration.\n";
+    print STDERR "--\n";
+    print STDERR "--           Total                                               Reads                 Olaps Evidence\n";
+    print STDERR "--    Job   Memory      Read Range         Reads        Bases   Memory        Olaps   Memory   Memory  (Memory in MB)\n";
+    print STDERR "--   ---- -------- ------------------- --------- ------------ -------- ------------ -------- --------\n";
 
     my $reads    = 0;
     my $bases    = 0;
     my $olaps    = 0;
 
-    my $coverage = getExpectedCoverage("unitigging", $asm);
+    my @bgn;
+    my @end;
+    my $nj = 0;
 
     push @bgn, 1;
 
     for (my $id = 1; $id <= $maxID; $id++) {
-        $reads += 1;
-        $bases += $readLengths[$id];
-        $olaps += $numOlaps[$id];
+        if (vec($rlVec, $id, 32) > 0) {
+            $reads += 1;
+            $bases += vec($rlVec, $id, 32);
+            $olaps += vec($noVec, $id, 32);
+        }
 
-        #  Guess how much extra memory used for overlapping reads.  Small genomes tend to load every read in the store,
-        #  large genomes ... load repeats + 2 * coverage * bases in reads (times 2 for overlaps off of each end)
+        #  Memory usage:
+        #
+        #  Per base/vote:
+        #    1 byte  for sequence
+        #   12 bytes for Vote_Tally_t
+        #
+        #  Per read:
+        #   32 bytes for Frag_Info_t
+        #
+        #  Per olap:
+        #   12 bytes for Olap_Info_t
+        #
+        #  When processing, the overlapping reads are loaded in batches of up to 100,000 reads
+        #  (depending on overlaps).  This needs 1 byte per base, but we don't know how many reads
+        #  are getting loaded, so we overestimate by finding the largest block of 100,000 reads that
+        #  could be loaded (done above) and using 2x that (because there are two buffers of these
+        #  reads).
+        #
+        #  Throw in another 2 GB for unknown overheads (gkpStore, ovlStore) and alignment generation.
 
-        my $memory = (13 * $bases) + (12 * $olaps) + (2 * $bases * $coverage);
+        my $memory = (12 * $bases) + (33 * $reads) + (12 * $olaps) + (2 * $maxBlockSize) + 2 * 1024 * 1024 * 1024;
 
-        if ((($maxMem   > 0) && ($memory >= $maxMem * 0.75)) ||    #  Allow 25% slop (10% is probably sufficient)
-            (($maxReads > 0) && ($reads  >= $maxReads))      ||
-            (($maxBases > 0) && ($bases  >= $maxBases))      ||
+        if ((($maxMem   > 0) && ($memory >= $maxMem))    ||
+            (($maxReads > 0) && ($reads  >= $maxReads))  ||
+            (($maxBases > 0) && ($bases  >= $maxBases))  ||
             (($id == $maxID))) {
             push @end, $id;
 
-            printf(STDERR "RED job %3u from read %9u to read %9u - %7.3f GB for %7u reads - %7.3f GB for %9u olaps - %7.3f GB for evidence\n",
-                   $nj + 1, $bgn[$nj], $end[$nj],
-                   $memory / 1024 / 1024 / 1024, $reads,
-                   13 * $bases / 1024 / 1024 / 1024, $bases,
-                   12 * $olaps / 1024 / 1024 / 1024, $olaps,
-                   2 * $bases * $coverage / 1024 / 1024 / 1024);
+            printf(STDERR "--   %4u %8.2f %9u-%-9u %9u %12u %8.2f %12u %8.2f %8.2f\n",
+                   $nj + 1,
+                   $memory / 1024 / 1024,
+                   $bgn[$nj], $end[$nj],
+                   $reads,
+                   $bases,               (12 * $bases + 33 * $reads)  / 1024 / 1024,
+                   $olaps,               (12 * $olaps)                / 1024 / 1024,
+                   2 * $maxBlockSize / 1024 / 1024);
 
             $nj++;
 
@@ -157,13 +221,15 @@ sub readErrorDetectionConfigure ($) {
         }
     }
 
+    print  STDERR "--   ---- -------- ------------------- --------- ------------ -------- ------------ -------- --------\n";
+    printf(STDERR "--                                               %12u          %12u\n",
+           $rlSum, $noSum);
+
     #  Dump a script.
 
     my $batchSize   = getGlobal("redBatchSize");
     my $numThreads  = getGlobal("redThreads");
 
-    my $numReads    = getNumberOfReadsInStore("unitigging", $asm);
-
     open(F, "> $path/red.sh") or caExit("can't open '$path/red.sh' for writing: $!", undef);
 
     print F "#!" . getGlobal("shell") . "\n\n";
@@ -211,7 +277,6 @@ sub readErrorDetectionConfigure ($) {
 
   finishStage:
     emitStage($asm, "readErrorDetectionConfigure");
-    buildHTML($asm, "utg");
 
   allDone:
 }
@@ -282,7 +347,6 @@ sub readErrorDetectionCheck ($) {
         #  Otherwise, run some jobs.
 
         emitStage($asm, "readErrorDetectionCheck", $attempt);
-        buildHTML($asm, "utg");
 
         submitOrRunParallelJob($asm, "red", $path, "red", @failedJobs);
         return;
@@ -326,7 +390,6 @@ sub readErrorDetectionCheck ($) {
     }
 
     emitStage($asm, "readErrorDetectionCheck");
-    buildHTML($asm, "utg");
 
   allDone:
 }
@@ -349,27 +412,10 @@ sub overlapErrorAdjustmentConfigure ($) {
     goto allDone   if (fileExists("unitigging/$asm.ovlStore/evalues"));   #  Stage entrely finished
     goto allDone   if (-d "unitigging/$asm.ctgStore");                    #  Assembly finished
 
-    #  OEA uses 1 byte/base + 8 bytes/adjustment + 28 bytes/overlap.  We don't know the number of adjustments, but that's
-    #  basically error rate.  No adjustment is output for mismatches.
+    my $maxID = getNumberOfReadsEarliestVersion($asm);
 
-    my @readLengths;
-    my @numOlaps;
-
-    #print STDERR "$bin/gatekeeperDumpMetaData -G unitigging/$asm.gkpStore -reads\n";
-    open(F, "$bin/gatekeeperDumpMetaData -G unitigging/$asm.gkpStore -reads |");
-    while (<F>) {
-        my @v = split '\s+', $_;
-        $readLengths[$v[0]] = $v[2];
-    }
-    close(F);
-
-    #print STDERR "$bin/ovStoreDump -G unitigging/$asm.gkpStore -O unitigging/$asm.ovlStore -d -counts\n";
-    open(F, "$bin/ovStoreDump -G unitigging/$asm.gkpStore -O unitigging/$asm.ovlStore -d -counts |");
-    while (<F>) {
-        my @v = split '\s+', $_;
-        $numOlaps[$v[0]] = $v[1];
-    }
-    close(F);
+    my ($rlVec, $noVec);
+    my ($rlSum, $noSum) = loadReadLengthsAndNumberOfOverlaps($asm, $maxID, \$rlVec, \$noVec);
 
     #  Make an array of partitions, putting as many reads into each as will fit in the desired memory.
 
@@ -380,13 +426,18 @@ sub overlapErrorAdjustmentConfigure ($) {
 
     my $nj = 0;
 
-    my $maxID    = getNumberOfReadsInStore("unitigging", $asm);
+    # get earliest count of reads in store
+    my $maxID    = getNumberOfReadsEarliestVersion($asm);
+
     my $maxMem   = getGlobal("oeaMemory") * 1024 * 1024 * 1024;
     my $maxReads = getGlobal("oeaBatchSize");
     my $maxBases = getGlobal("oeaBatchLength");
 
-    print STDERR "\n";
-    print STDERR "Configure OEA for ", getGlobal("oeaMemory"), "gb memory with batches of at most ", ($maxReads > 0) ? $maxReads : "(unlimited)", " reads and ", ($maxBases > 0) ? $maxBases : "(unlimited)", " bases.\n";
+    print STDERR "--\n";
+    print STDERR "-- Configure OEA for ", getGlobal("oeaMemory"), "gb memory.\n";
+    print STDERR "--                   Batches of at most ", ($maxReads > 0) ? $maxReads : "(unlimited)", " reads.\n";
+    print STDERR "--                                      ", ($maxBases > 0) ? $maxBases : "(unlimited)", " bases.\n";
+    print STDERR "--\n";
 
     my $reads    = 0;
     my $bases    = 0;
@@ -394,7 +445,6 @@ sub overlapErrorAdjustmentConfigure ($) {
 
     fetchFile("$path/red.red");
 
-    my $coverage     = getExpectedCoverage("unitigging", $asm);
     my $corrSize     = (-s "$path/red.red");
 
     my $smallJobs    = 0;
@@ -403,42 +453,48 @@ sub overlapErrorAdjustmentConfigure ($) {
     push @bgn, 1;
 
     for (my $id = 1; $id <= $maxID; $id++) {
-        $reads += 1;
-        $bases += $readLengths[$id];
-        $olaps += $numOlaps[$id];
+        if (vec($rlVec, $id, 32) > 0) {
+            $reads += 1;
+            $bases += vec($rlVec, $id, 32);
+            $olaps += vec($noVec, $id, 32);
+        }
+
+        #  OEA uses 1 byte/base + 8 bytes/adjustment + 28 bytes/overlap.  We don't know the number
+        #  of adjustments, but that's basically error rate.  No adjustment is output for mismatches.
 
         #  Hacked to attempt to estimate adjustment size better.  Olaps should only require 12 bytes each.
 
-        my $memBases  = (1   * $bases);              #  Corrected reads for this batch
-        my $memAdj1   = (8   * $corrSize) * 0.33;    #  Overestimate of the size of the indel adjustments needed (total size includes mismatches)
-        my $memReads  = (32  * $reads);              #  Read data in the batch
-        my $memOlaps  = (32  * $olaps);              #  Loaded overlaps
-        my $memSeq    = (4   * 2097152);             #  two char arrays of 2*maxReadLen
-        my $memAdj2   = (16  * 2097152);             #  two Adjust_t arrays of maxReadLen
-        my $memWA     = (32  * 1048576);             #  Work area (16mb) and edit array (16mb)
-        my $memMisc   = (256 * 1048576);             #  Work area (16mb) and edit array (16mb) and (192mb) slop
+        my $memBases  = (1    * $bases);              #  Corrected reads for this batch
+        my $memAdj1   = (8    * $corrSize) * 0.33;    #  Overestimate of the size of the indel adjustments needed (total size includes mismatches)
+        my $memReads  = (32   * $reads);              #  Read data in the batch
+        my $memOlaps  = (32   * $olaps);              #  Loaded overlaps
+        my $memSeq    = (4    * 2097152);             #  two char arrays of 2*maxReadLen
+        my $memAdj2   = (16   * 2097152);             #  two Adjust_t arrays of maxReadLen
+        my $memWA     = (32   * 1048576);             #  Work area (16mb) and edit array (16mb)
+        my $memMisc   = (256  * 1048576);             #  Work area (16mb) and edit array (16mb) and (192mb) slop
+        my $memExtra  = (2048 * 1048576);             #  For alignments and overhead.
 
-        my $memory = $memBases + $memAdj1 + $memReads + $memOlaps + $memSeq + $memAdj2 + $memWA + $memMisc;
+        my $memory = $memBases + $memAdj1 + $memReads + $memOlaps + $memSeq + $memAdj2 + $memWA + $memMisc + $memExtra;
 
-        if ((($maxMem   > 0) && ($memory >= $maxMem * 0.75)) ||
-            (($maxReads > 0) && ($reads  >= $maxReads))      ||
-            (($maxBases > 0) && ($bases  >= $maxBases))      ||
+        if ((($maxMem   > 0) && ($memory >= $maxMem))   ||
+            (($maxReads > 0) && ($reads  >= $maxReads)) ||
+            (($maxBases > 0) && ($bases  >= $maxBases)) ||
             (($id == $maxID))) {
             push @end, $id;
 
             $smallJobs++   if ($end[$nj] - $bgn[$nj] < $smallJobSize);
 
-            push @log, sprintf("OEA job %3u from read %9u to read %9u - %4.1f bases + %4.1f adjusts + %4.1f reads + %4.1f olaps + %4.1f fseq/rseq + %4.1f fadj/radj + %4.1f work + %4.1f misc = %5.1f MB\n",
-                   $nj + 1, $bgn[$nj], $end[$nj],
-                   $memBases / 1024 / 1024,
-                   $memAdj1  / 1024 / 1024,
-                   $memReads / 1024 / 1024,
-                   $memOlaps / 1024 / 1024,
-                   $memSeq   / 1024 / 1024,
-                   $memAdj2  / 1024 / 1024,
-                   $memWA    / 1024 / 1024,
-                   $memMisc  / 1024 / 1024,
-                   $memory   / 1024 / 1024);
+            #  Save the log for later printing.  We redo the configuration if there are too many small jobs.
+
+            push @log, sprintf("--   %4u %8.2f %9u-%-9u %9u %12u %8.2f %12u %8.2f %8.2f\n",
+                               $nj + 1,
+                               $memory / 1024 / 1024,
+                               $bgn[$nj], $end[$nj],
+                               $reads, $bases,
+                               ($memReads + $memBases + $memSeq) / 1024 / 1024,
+                               $olaps,
+                               $memOlaps / 1024 / 1024,
+                               ($memAdj1 + $memAdj2 + $memWA + $memMisc) / 1024 / 1024);
 
             $nj++;
 
@@ -466,14 +522,17 @@ sub overlapErrorAdjustmentConfigure ($) {
 
     #  Report.
 
-    print STDERR "Configured $nj jobs.\n";
-    print STDERR "\n";
+    print STDERR "--           Total                                               Reads                 Olaps  Adjusts\n";
+    print STDERR "--    Job   Memory      Read Range         Reads        Bases   Memory        Olaps   Memory   Memory  (Memory in MB)\n";
+    print STDERR "--   ---- -------- ------------------- --------- ------------ -------- ------------ -------- --------\n";
 
     foreach my $l (@log) {
         print STDERR $l;
     }
 
-    print STDERR "\n";
+    print  STDERR "--   ---- -------- ------------------- --------- ------------ -------- ------------ -------- --------\n";
+    printf(STDERR "--                                               %12u          %12u\n",
+           $rlSum, $noSum);
 
     #  Dump a script
 
@@ -526,7 +585,6 @@ sub overlapErrorAdjustmentConfigure ($) {
 
   finishStage:
     emitStage($asm, "overlapErrorAdjustmentConfigure");
-    buildHTML($asm, "utg");
 
   allDone:
 }
@@ -553,8 +611,7 @@ sub overlapErrorAdjustmentCheck ($) {
 
     my $batchSize   = getGlobal("oeaBatchSize");
     my $failedJobs  = 0;
-    my $numReads    = getNumberOfReadsInStore("unitigging", $asm);
-    my $numJobs     = 0;  #int($numReads / $batchSize) + (($numReads % $batchSize == 0) ? 0 : 1);
+    my $numJobs     = 0;
 
     #  Need to read script to find number of jobs!
 
@@ -603,7 +660,6 @@ sub overlapErrorAdjustmentCheck ($) {
         #  Otherwise, run some jobs.
 
         emitStage($asm, "overlapErrorAdjustmentCheck", $attempt);
-        buildHTML($asm, "utg");
 
         submitOrRunParallelJob($asm, "oea", $path, "oea", @failedJobs);
         return;
@@ -621,7 +677,6 @@ sub overlapErrorAdjustmentCheck ($) {
     stashFile("$path/oea.files");
 
     emitStage($asm, "overlapErrorAdjustmentCheck");
-    buildHTML($asm, "utg");
 
   allDone:
 }
@@ -680,7 +735,6 @@ sub updateOverlapStore ($) {
 
   finishStage:
     emitStage($asm, "updateOverlapStore");
-    buildHTML($asm, "utg");
 
   allDone:
 }
